#!/bin/bash

# Repo Master Parameters
masterHostname=azure-apt-cat.cloudapp.net
masterReporoot=/home/azureuser/apt-catalog/repo_build/repos
masterAccounts=/home/azureuser/apt-catalog/repo_build/repo_accounts.csv

# Local Parameters
localUser=azureuser
tmpAccounts=/home/$localUser/repo_accounts.csv
accounts=/etc/azuremirror/repo_accounts.csv
reporoot=/var/reporoot
sshdConfig=/etc/ssh/sshd_config
sshKey=/home/$localUser/.ssh/id_rsa

newAccts=0
runningUser=$USER
syslogFacility=syslog
syslogTag=SyncMirror

# Function Definitions
function LogMessage {
    msg=$1
    priority=$2
    if [ -z "$msg" ];then
        return
    fi
    
    if [ -z "$priority" ]; then
        priority=info
    fi
    
    logger -p "$syslogFacility.$priority" -t $SyncMirror -- $msg
    echo "[$priority]: $msg"
}

function Bail {
    msg=$1
    if [ -z "$msg" ];then
        msg="An unspecified error occurred"
    fi
    LogMessage $msg error
    exit 1
}

# Pre-execution check
runningUser=$SUDO_USER
if [[ -z $SUDO_USER && "$USER" -ne "root" ]];then
    Bail "Program must run under sudo or root"
fi

# Get CSV file
sudo -u $localUser scp $masterHostname:$masterAccounts $tmpAccounts

if [[ $? -ne 0 || ! -f $tmpAccounts ]]; then
    Bail "File $tmpAccounts was not successfully copied from master"
fi

# Move CSV file to a location where users cannot mettle with it
if [ ! -f /etc/azuremirror ]; then
    mkdir -p /etc/azuremirror
fi

mv $tmpAccounts $accounts
if [ -f $accounts ]; then
    LogMessage "Successfully retrieved $accounts"
else
    Bail "Failed to move $tmpAccounts to $accounts"
fi
chown root:root $accounts
chmod 600 $accounts

# Make repo folders if they don't already exist
if [ ! -f $reporoot/repos ]; then
    LogMessage "Created $reporoot/repos"
    mkdir -p $reporoot/repos
fi
LogMessage "Setting root:root as owner of $reporoot"
chown root:root $reporoot
LogMessage "Setting $localUser as owner of $reporoot/repos"
chown $localUser:$localUser $reporoot/repos
chmod 755 $reporoot $reporoot/repos

while IFS=, read username groups folder sshkey
do
    if [ -z $username ];then
        continue
    fi
    acctExists=$(grep -c "^$username:" /etc/passwd)
    # Setup account if it doesn't already exist
    if [ $acctExists -lt 1 ]; then
        # Set up account
        LogMessage "Account $username doesn't exist. Creating it."
        useradd -d /home/$username -m -s /bin/false $username
        mkdir -p /home/$username/.ssh
        # Set up ssh key
        echo "ssh-rsa $sshkey" > /home/$username/.ssh/authorized_keys
        chown -R $username:$username /home/$username/.ssh
        chmod 700 /home/$username/.ssh
        chmod 600 /home/$username/.ssh/authorized_keys
        # Set up SFTP permissions
        cat << EOF >> $sshdConfig

Match User $username
	ChrootDirectory $reporoot
	AllowTCPForwarding no
	X11Forwarding no
	ForceCommand internal-sftp

EOF
        let "newAccts += 1"
    fi
    # Update group membership
    groupsCmd=$(echo $groups | tr ':' ',')
    LogMessage "Setting group membership for $username to [$groupsCmd]"
    usermod -G "$groupsCmd" $username
done < $accounts

if [ $newAccts -gt 0 ]; then
    LogMessage "Added $newAccts accounts, must restart SSH daemon"
    service ssh restart >> /dev/null
    if [ $? -ne 0 ]; then
        Bail "Error while restarting SSH daemon"
    fi
    LogMessage "Successfully restarted SSH daemon"  
fi

# Rsync package files
startTime=$(date +%s)
rsync -agpz -e "ssh -i /home/$localUser/.ssh/id_rsa" $localUser@$masterHostname:$masterReporoot $reporoot
errCode=$?
endTime=$(date +%s)
elapsed=$((endTime-startTime))
if [ $errCode -ne 0 ];then
    Bail "Rsync failed after $elapsed seconds"
else
    LogMessage "Rsync succeeded after $elapsed seconds"
fi